import numpy as np
import cv2
    
image = cv2.imread("../sheet-on-table.png",cv2.IMREAD_COLOR)

cv2.imshow("original",image)
cv2.waitKey(0)

gray = cv2.cvtColor(image,cv2.COLOR_BGR2GRAY)

_, binary = cv2.threshold(gray,0,255,cv2.THRESH_BINARY | cv2.THRESH_OTSU)

cv2.imshow("binary",binary)
cv2.waitKey(0)

contours,_ = cv2.findContours(binary, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
rotatedRect = cv2.minAreaRect(contours[0])
print(rotatedRect)

points = cv2.boxPoints(rotatedRect)
for i in range(len(points)):
    cv2.line(binary,points[i].astype(np.int32),points[(i+1)%len(points)].astype(np.int32),127,2)
cv2.imshow("binary",binary)
cv2.waitKey(0)

angle = rotatedRect[2]
warp_matrix = cv2.getRotationMatrix2D((image.shape[1]/2,image.shape[0]/2),angle,1)
rotated = cv2.warpAffine(image,warp_matrix,(image.shape[1],image.shape[0]))
cv2.imshow("rotated",rotated)
cv2.waitKey(0)

rotatedPoints = np.intp(cv2.transform(np.array([points]), warp_matrix))
rect = cv2.boundingRect(rotatedPoints)
aligned = rotated[rect[1]:rect[1]+rect[3],rect[0]:rect[0]+rect[2]]

cv2.imshow("aligned",aligned)
cv2.waitKey(0)
cv2.destroyAllWindows()

cv2.imwrite("aligned.png",aligned)