#include <opencv2/opencv.hpp>

void phase(cv::Mat &src, cv::Mat &phase)
{
    cv::Mat dst(src.size(), src.type());
    for (int y = 0; y < src.rows; y++) {
        for (int x = 0; x < src.cols; x++) {
            cv::Vec2f v = src.at<cv::Vec2d>(y, x);
            dst.at<cv::Vec2d>(y, x) = v / std::sqrt(std::pow(v[0], 2.0) + std::pow(v[1], 2.0));
        }
    }
    phase = dst;
}

int main ()
{
    cv::Mat a0 = cv::imread("dom.png",cv::IMREAD_GRAYSCALE);
    cv::Mat b0 = cv::imread("okno.png",cv::IMREAD_GRAYSCALE);

    cv::Mat a0_f, b0_f;
    a0.convertTo(a0_f,CV_64F);
    b0.convertTo(b0_f,CV_64F);

    cv::Mat a = 1.0 - a0_f / 255.0, b(a.rows,a.cols,a.type(),cv::Scalar(0)), d;
    b(cv::Rect(0,0,b0.cols,b0.rows)) = 1.0 - b0_f / 255.0;

    cv::Mat fa_c, fb_c, fd_c;
    cv::dft(a,fa_c, cv::DFT_COMPLEX_OUTPUT);
    cv::dft(b,fb_c, cv::DFT_COMPLEX_OUTPUT);
    phase(fa_c,fa_c);
    phase(fb_c,fb_c);
    cv::mulSpectrums(fa_c, fb_c, fd_c, 0, true);
    cv::idft(fd_c,d, cv::DFT_REAL_OUTPUT | cv::DFT_SCALE);

    cv::Point shift;
    cv::minMaxLoc(d, NULL, NULL, NULL, &shift);
    printf("%d %d\n",shift.x,shift.y);

    cv::Mat res;
    cv::cvtColor(a0,res,cv::COLOR_GRAY2RGB);
    cv::rectangle(res,cv::Point(shift.x,shift.y),cv::Point(shift.x+b0.cols,shift.y+b0.rows),cv::Scalar(0,0,255),2);
    cv::imshow("found",res);
    cv::imshow("sought",b0);
    cv::waitKey(0);
    cv::destroyAllWindows();

    return 0;
}
